--[[
	ArcanePartyBars
	 	Tracks Party Casting via Sky and displays it on their party frame.
	
	By: AnduinLothar
	
	Special Thanks:
	Lelik (Healix): Initial Target Guessing code,
	Zlixar (ArcaneBar): Event and Casting Tecqunique,
	Mudgendai (CastOptions): Troubleshooting, Self/Friendly Targeting Spell Textures, container gathering code
	Legorol (EquipCompare): Troubleshooting, Timing and Theoretical Support
	
	
	$Id: ArcanePartyBars.lua 2802 2005-11-20 15:43:04Z mugendai $
	$Rev: 2802 $
	$LastChangedBy: mugendai $
	$Date: 2005-11-20 09:43:04 -0600 (Sun, 20 Nov 2005) $
	
	
	Spell Targeting wont be correct if you Queue a spell and then use any of the following Target functions:
	
    TargetNearestFriend(), TargetNearestPartyMember(), TargetNearestRaidMember() 
]]--

--------------------------------------------------
-- Globals
--------------------------------------------------
ArcanePartyBars = {};
ArcanePartyBars.DefaultColors = {
	["MAIN"] = {
		r=1.0;
		g=0.7;
		b=0.0;		
	};
	["CHANNEL"] = {
		r=0.0;
		g=1.0;
		b=0.0;
	};
	["SUCCESS"] = {
		r=0.0;
		g=1.0;
		b=0.0;
	};
	["FAILURE"] = {
		r=1.0;
		g=0.0;
		b=0.0;
	};	
};

ArcanePartyBars.CORPSE_TOOLTIP_REGEX = gsub(format(CORPSE_TOOLTIP, "0"), "(.*)0(.*)", "%1(.*)%2");
ArcanePartyBars.SPELL_CAST_TIME_SEC_REGEX = gsub(format(SPELL_CAST_TIME_SEC, 0), "(.*)0(.*)", "%1(.*)%2");
ArcanePartyBars.SPELL_RANGE_REGEX = gsub(format(SPELL_RANGE, 0), "(.*)0(.*)", "%1%%d%2");
ArcanePartyBars.TIME_LEFT = "%.1fs";

ArcanePartyBars_OnUpdateSimulationPeriod = .07;

ArcanePartyBars_Enabled = true;
ArcanePartyBars_Mobile = false;
ArcanePartyBars_SimulateOnUpdate = false;
ArcanePartyBars_SendCastingEvents = true;
ArcanePartyBars_ShowPartyCasting = true;
ArcanePartyBars_AutoJoinSkyParty = true;

ArcanePartyBarDebugParty1 = false;

--------------------------------------------------
-- Configuration Functions
--------------------------------------------------

--ArcanePartyBars[ArcanePartyBars.ConfigTable[var][getglobal(var)]]()
ArcanePartyBars.ConfigTable = {
	ArcanePartyBars_Enabled = {[1]="Enable", [0]="Disable", text="MASTER_ENABLE", id="ArcanePartyBars", default=true, disabled=false, difficulty=1, com="/apb"};
	ArcanePartyBars_Mobile = {[1]="Show", [0]="Hide", text="MAKE_DRAGGABLE", id="MakeMobile", default=false, disabled=false, difficulty=2, subcom="mobile"};
	ArcanePartyBars_SimulateOnUpdate = {[1]="SimulateOnUpdate", [0]="EnableOnUpdate", text="SIMULATE_ONUPDATE", id="SimulateOnUpdate", default=false, disabled=false, difficulty=4, subcom="sim"};
	ArcanePartyBars_TopText = {[1]="TopText", [0]="RightText", text="TOP_TEXT", id="TopText", default=false, disabled=false, difficulty=2, subcom="toptext"};
	ArcanePartyBars_SendCastingEvents = {[1]="emptyfunc", [0]="emptyfunc", text="SEND_CASTS", id="SendCasts", default=true, disabled=false, difficulty=3, subcom="sendcasts"};
	ArcanePartyBars_ShowPartyCasting = {[1]="emptyfunc", [0]="emptyfunc", text="SHOW_CASTS", id="ShowCasts", default=true, disabled=false, difficulty=3, subcom="showcasts"};
	ArcanePartyBars_AutoJoinSkyParty = {[1]="AttemptToJoinSkyParty", [0]="emptyfunc", text="AUTO_JOIN", id="AutoJoin", default=true, disabled=false, difficulty=2, subcom="autojoin"};
};

ArcanePartyBars.Enable = function()
	ArcanePartyBars.Register(ArcanePlayerBarsFrame);
	ArcanePartyBars.RegisterForSkyMessages();
	ArcanePartyBars.Hide();
	ArcanePartyBars_Enabled = true;
end

ArcanePartyBars.Disable = function()
	ArcanePartyBars.Unregister(ArcanePlayerBarsFrame);
	ArcanePartyBars.UnregisterForSkyMessages();
	ArcanePartyBars.Hide();
	ArcanePartyBars_Enabled = false;
end

ArcanePartyBars.ToggleMobility = function()
	if (ArcanePartyBars_Mobile) then
		ArcanePartyBars.Hide();
	else
		ArcanePartyBars.Show();
	end
end

ArcanePartyBars.Register = function(frame)
	if (not frame) then
		frame = this;
	end
	frame:RegisterEvent("SPELLCAST_START");
	frame:RegisterEvent("SPELLCAST_STOP");
	frame:RegisterEvent("SPELLCAST_FAILED");
	frame:RegisterEvent("SPELLCAST_INTERRUPTED");
	frame:RegisterEvent("SPELLCAST_DELAYED");
	frame:RegisterEvent("SPELLCAST_CHANNEL_START");
	frame:RegisterEvent("SPELLCAST_CHANNEL_UPDATE");
	frame:RegisterEvent("PLAYER_TARGET_CHANGED");
	frame:RegisterEvent("PARTY_MEMBERS_CHANGED");
	Sea.util.hook("UseContainerItem", "ArcanePartyBars.UseContainerItem_Hook", "before");
	Sea.util.hook("CastSpell", "ArcanePartyBars.CastSpell_Hook", "before");
	Sea.util.hook("UseAction", "ArcanePartyBars.UseAction_Hook", "before");
	Sea.util.hook("CameraOrSelectOrMoveStop","ArcanePartyBars.CameraOrSelectOrMoveStop_Hook","before");
	Sea.util.hook("SpellTargetUnit","ArcanePartyBars.TargetUnit_Hook", "before");
	Sea.util.hook("TargetUnit","ArcanePartyBars.TargetUnit_Hook", "before");
	Sea.util.hook("TargetByName","ArcanePartyBars.TargetByName_Hook", "before");
	Sea.util.hook("TargetLastTarget","ArcanePartyBars.TargetLastTarget_Hook", "before");
	if (CastOptions) and (CastOptions.CastAtUnit) then
		Sea.util.hook("CastOptions.CastAtUnit","ArcanePartyBars.TargetUnit_Hook", "after");
	end
	Sea.util.hook("TargetNearestFriend","ArcanePartyBars.TargetUnknown_Hook", "before");
	Sea.util.hook("TargetNearestPartyMember","ArcanePartyBars.TargetUnknown_Hook", "before");
	Sea.util.hook("TargetNearestRaidMember","ArcanePartyBars.TargetUnknown_Hook", "before");
end
-- Unregisters "frame" (passed as a string) from spellcast events.
ArcanePartyBars.Unregister = function(frame)
	if (not frame) then
		frame = this;
	end
	frame:UnregisterEvent("SPELLCAST_START");
	frame:UnregisterEvent("SPELLCAST_STOP");
	frame:UnregisterEvent("SPELLCAST_FAILED");
	frame:UnregisterEvent("SPELLCAST_INTERRUPTED");
	frame:UnregisterEvent("SPELLCAST_DELAYED");
	frame:UnregisterEvent("SPELLCAST_CHANNEL_START");
	frame:UnregisterEvent("SPELLCAST_CHANNEL_UPDATE");
	frame:UnregisterEvent("PLAYER_TARGET_CHANGED");
	frame:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	Sea.util.unhook("UseContainerItem", "ArcanePartyBars.UseContainerItem_Hook", "before");
	Sea.util.unhook("CastSpell", "ArcanePartyBars.CastSpell_Hook", "before");
	Sea.util.unhook("UseAction", "ArcanePartyBars.UseAction_Hook", "before");
	Sea.util.unhook("CameraOrSelectOrMoveStop","ArcanePartyBars.CameraOrSelectOrMoveStop_Hook","before");
	Sea.util.unhook("SpellTargetUnit","ArcanePartyBars.TargetUnit_Hook", "before");
	Sea.util.unhook("TargetUnit","ArcanePartyBars.TargetUnit_Hook", "before");
	Sea.util.unhook("TargetByName","ArcanePartyBars.TargetByName_Hook", "before");
	Sea.util.unhook("TargetLastTarget","ArcanePartyBars.TargetLastTarget_Hook", "before");
	if (CastOptions) and (CastOptions.CastAtUnit) then
		Sea.util.unhook("CastOptions.CastAtUnit","ArcanePartyBars.TargetUnit_Hook", "after");
	end
	Sea.util.unhook("TargetNearestFriend","ArcanePartyBars.TargetUnknown_Hook", "before");
	Sea.util.unhook("TargetNearestPartyMember","ArcanePartyBars.TargetUnknown_Hook", "before");
	Sea.util.unhook("TargetNearestRaidMember","ArcanePartyBars.TargetUnknown_Hook", "before");
end

ArcanePartyBars.Reset = function(bar)
	bar:ClearAllPoints();
	bar:SetPoint("BOTTOMLEFT", bar:GetParent(), "BOTTOMRIGHT", 7, 43);
	bar:SetUserPlaced(0);
end

ArcanePartyBars.ResetAll = function()
	for i=1, 4 do
		ArcanePartyBars.Reset(getglobal("ArcanePartyBar"..i));
	end
end

ArcanePartyBars.Show = function()
	local bar;
	for i=1, 4 do
		bar = getglobal("ArcanePartyBar"..i);
		bar:Show();
		bar:SetAlpha(1);
		bar:EnableMouse(1);
		bar.Name:SetText(ArcanePartyBars.localization.BAR_TITLES[1]);
		bar.Time:SetText("(10.0)");
		bar.Target:SetText("<"..UnitName("player")..">");
	end
	ArcanePartyBars_Mobile = true;
end

ArcanePartyBars.Hide = function()
	local bar;
	for i=1, 4 do
		bar = getglobal("ArcanePartyBar"..i);
		bar:Hide();
		bar:SetAlpha(0);
		bar:EnableMouse(0);
		bar.Name:SetText("");
		bar.Time:SetText("");
		bar.Target:SetText("");
	end
	ArcanePartyBars_Mobile = false;
end

ArcanePartyBars.TopText = function()
	local bar;
	for i=1, 4 do
		bar = getglobal("ArcanePartyBar"..i);
		bar.Target:ClearAllPoints();
		bar.Target:SetPoint("BOTTOM", bar, "TOP", -10, 5);
		bar.Time:ClearAllPoints();
		bar.Time:SetPoint("LEFT", bar.Target, "RIGHT", 3, 0);
	end
	ArcanePartyBars_TopText = true;
end

ArcanePartyBars.RightText = function()
	local bar;
	for i=1, 4 do
		bar = getglobal("ArcanePartyBar"..i);
		bar.Time:ClearAllPoints();
		bar.Time:SetPoint("LEFT", bar, "RIGHT", 7, 0);
		bar.Target:ClearAllPoints();
		bar.Target:SetPoint("LEFT", bar.Time, "RIGHT", 3, 0);
	end
	ArcanePartyBars_TopText = false;
end

ArcanePartyBars.ToggleTextLocation = function()
	if (ArcanePartyBars_TopText) then
		ArcanePartyBars.RightText();
	else
		ArcanePartyBars.TopText();
	end
end

ArcanePartyBars.SimulateOnUpdate = function()
	if (not Chronos.scheduleRepeating) then
		Sea.io.print(ArcanePartyBars.localization.SIMULATION_FAILURE);
		return;
	end
	for i=1, 4 do
		local  bar = getglobal("ArcanePartyBar"..i);
		if (bar:IsVisible()) then
			Chronos.scheduleRepeating(bar:GetName().."_OnUpdateSimulator", ArcanePartyBars_OnUpdateSimulationPeriod, function() ArcanePartyBars.OnUpdateEvaluation(bar) end);
		end
	end
	ArcanePartyBars_SimulateOnUpdate = true;
end

ArcanePartyBars.EnableOnUpdate = function()
	local bar;
	for i=1, 4 do
		local bar = getglobal("ArcanePartyBar"..i);
		Chronos.unscheduleRepeating(bar:GetName().."_OnUpdateSimulator");
	end
	ArcanePartyBars_SimulateOnUpdate = false;
end

ArcanePartyBars.ToggleOnUpdateSimulation = function()
	if (ArcanePartyBars_SimulateOnUpdate) then
		ArcanePartyBars.EnableOnUpdate();
	else
		ArcanePartyBars.SimulateOnUpdate();
	end
end

ArcanePartyBars.ShowPartyCasting = function()
	ArcanePartyBars.RegisterForSkyMessages()
	ArcanePartyBars_ShowPartyCasting = true;
end

ArcanePartyBars.HidePartyCasting = function()
	ArcanePartyBars.UnregisterForSkyMessages()
	ArcanePartyBars_ShowPartyCasting = false;
end

ArcanePartyBars.TogglePartyCasting = function()
	if (ArcanePartyBars_ShowPartyCasting) then
		ArcanePartyBars.HidePartyCasting();
	else
		ArcanePartyBars.ShowPartyCasting();
	end
end

ArcanePartyBars.MoveTextToTop = function()
	local frame;
	for i=1, 4 do
		local frame = getglobal("ArcanePartyBar"..i);
		frame.Target:ClearAllPoints();
		frame.Target:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 0, 0);
		frame.Time:ClearAllPoints();
		frame.Time:SetPoint("LEFT", frame.Target, "RIGHT", 3, 0);
	end
	ArcanePartyBars_TextOnTop_Enabled = true;
end

ArcanePartyBars.MoveTextToSide = function()
	local frame;
	for i=1, 4 do
		local frame = getglobal("ArcanePartyBar"..i);
		frame.Time:ClearAllPoints();
		frame.Time:SetPoint("LEFT", frame, "RIGHT", 7, 0);
		frame.Target:ClearAllPoints();
		frame.Target:SetPoint("LEFT", frame.Time, "RIGHT", 3, 0);
	end
	ArcanePartyBars_TextOnTop_Enabled = false;
end

ArcanePartyBars.ToggleMoveText = function()
	if (ArcanePartyBars_TextOnTop_Enabled) then
		ArcanePartyBars.MoveTextToSide();
	else
		ArcanePartyBars.MoveTextToTop();
	end
end

ArcanePartyBars.AttemptToJoinSkyParty = function()
	if (ArcanePartyBars_AutoJoinSkyParty) and (UnitInParty("party1")) then
		if ( not Sky.isChannelActive(SKY_PARTY) ) then
			if (SkyChannelManager.getChannelCount() < 10) then
				JoinChannelByName(SKY_PARTY, nil, nil, true);
			elseif ( not ArcanePartyBars_JoinPartyAlert ) then 
				Sea.io.print(ArcanePartyBars.localization.AUTO_JOIN_ALERT); 
				ArcanePartyBars_JoinPartyAlert = true;
			end
		end
	end
end

--------------------------------------------------
-- Bar Scripts
--------------------------------------------------

ArcanePartyBars.OnLoad = function(frame)
	if (not frame) then
		frame = this;
	end
	frame.StatusBar = getglobal(frame:GetName().."StatusBar");
	frame.StatusBar:EnableMouse(0);
	frame:EnableMouse(0);
	frame.Spark = getglobal(frame.StatusBar:GetName().."Spark");
	frame.Name = getglobal(frame.StatusBar:GetName().."Name");
	frame.Time = getglobal(frame.StatusBar:GetName().."Time");
	frame.Time:ClearAllPoints();
	frame.Time:SetPoint("LEFT", frame, "RIGHT", 7, 0);
	frame.Target = getglobal(frame.StatusBar:GetName().."Target");
	frame.Target:ClearAllPoints();
	frame.Target:SetPoint("LEFT", frame.Time, "RIGHT", 3, 0);
	frame.Flash = getglobal(frame:GetName().."Flash");
	frame.StatusBar:SetFrameLevel(frame.StatusBar:GetFrameLevel() - 1);
	frame.Flash:SetFrameLevel(frame.StatusBar:GetFrameLevel() + 1);
	frame.casting = nil;
	frame.holdTime = 0;
	frame.Enabled = 0;
	frame.Overrided = 0;
	
	--Simulate OnUpdate (moved to OnShow)
	--Chronos.scheduleRepeating(frame:GetName().."_OnUpdateSimulator", ArcanePartyBars_OnUpdateSimulationPeriod, function() ArcanePartyBars.OnUpdateEvaluation(frame) end);
end

ArcanePartyBars.OnUpdate = function(frame)
	if (not ArcanePartyBars_SimulateOnUpdate) then
		ArcanePartyBars.OnUpdateEvaluation(frame);
	end
end

ArcanePartyBars.OnUpdateEvaluation = function(frame)
	if (not frame) then
		frame = this;
	end
	if ( ArcanePartyBars_ShowPartyCasting ) then
		if ( frame.channeling ) then
			local time = GetTime();
			if ( time > frame.endTime ) then
				time = frame.endTime
			end
			if ( time == frame.endTime ) then
				frame.channeling = nil;
				frame.fadeOut = 1;
				return;
			end
			local barValue = frame.startTime + (frame.endTime - time);
			frame.StatusBar:SetValue( barValue );
			frame.Flash:Hide();
			local sparkPosition = ((barValue - frame.startTime) / (frame.endTime - frame.startTime)) * frame:GetWidth();
			frame.Spark:SetPoint("CENTER", frame.StatusBar, "LEFT", sparkPosition, 0);
			frame.Time:SetText("("..ArcanePartyBars.GetTimeLeft(frame.StatusBar)..")");
		elseif ( frame.casting ) then
			local status = GetTime();
			if ( status > frame.maxValue ) then
				status = frame.maxValue
			end
			frame.StatusBar:SetValue(status);
			frame.Flash:Hide();
			local sparkPosition = ((status - frame.startTime) / (frame.maxValue - frame.startTime)) * frame:GetWidth();
			if ( sparkPosition < 0 ) then
				sparkPosition = 0;
			end
			frame.Spark:SetPoint("CENTER", frame.StatusBar, "LEFT", sparkPosition, 0);
			frame.Time:SetText("("..ArcanePartyBars.GetTimeLeft(frame.StatusBar)..")");
			if ( status == frame.maxValue ) then
				--ArcanePartyBars.SpellcastStop(frame);
			end
		elseif ( GetTime() < frame.holdTime ) then
			return;
		elseif ( frame.flash ) then
			local alpha = frame.Flash:GetAlpha() + CASTING_BAR_FLASH_STEP;
			if ( alpha < 1 ) then
				frame.Flash:SetAlpha(alpha);
			else
				frame.flash = nil;
			end
		elseif ( frame.fadeOut ) then
			local alpha = frame:GetAlpha() - CASTING_BAR_ALPHA_STEP;
			if ( alpha > 0 ) then
				frame:SetAlpha(alpha);
			else
				frame.fadeOut = nil;
				frame:Hide();
			end
		end
	end
end

ArcanePartyBars.OnEnter = function(frame)
	if (not frame) then
		frame = this;
	end
	if (ArcanePartyBars_Mobile) then
		APB_Tooltip:SetOwner(frame, "ANCHOR_CURSOR");
		APB_Tooltip:SetText(format(ArcanePartyBars.localization.TOOLTIP_TEXT, ArcanePartyBars.localization.BAR_TITLES[frame:GetID()]));
	end
end

ArcanePartyBars.OnLeave = function(frame)
	if (not frame) then
		frame = this;
	end
	APB_Tooltip:Hide();
end

ArcanePartyBars.OnMouseUp = function(frame)
	if (not frame) then
		frame = this;
	end
	if ( frame.isMoving ) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
	end
end

ArcanePartyBars.OnMouseDown = function(frame)
	if (not frame) then
		frame = this;
	end
	if (arg1 == "RightButton") and (IsShiftKeyDown()) then
		APB_DropDown.displayMode = "MENU";
		ToggleDropDownMenu(1, frame:GetID(), APB_DropDown, frame:GetName());
	elseif (arg1 == "LeftButton") then
		if (not frame.isLocked) or (frame.isLocked == 0) then
			frame:StartMoving();
			frame.isMoving = true;
			APB_Tooltip:Hide();
		end
	end
end

ArcanePartyBars.OnShow = function(frame)
	if (not frame) then
		frame = this;
	end
	if (ArcanePartyBars_SimulateOnUpdate) then
		--Simulate OnUpdate
		Chronos.scheduleRepeating("ArcanePartyBar1_OnUpdateSimulator", ArcanePartyBars_OnUpdateSimulationPeriod, function() ArcanePartyBars.OnUpdateEvaluation(ArcanePartyBar1) end);
	end
end

ArcanePartyBars.OnHide = function(frame)
	if (not frame) then
		frame = this;
	end
	if (Chronos.unscheduleRepeating) then
		Chronos.unscheduleRepeating(frame:GetName().."_OnUpdateSimulator");
	end
	if ( frame.isMoving ) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
	end
end


--------------------------------------------------
-- Bar Functions
--------------------------------------------------

ArcanePartyBars.GetTimeLeft = function(statusBar)
	local min, max = statusBar:GetMinMaxValues();
	local current_time;
	if ( this.channeling ) then
		current_time = statusBar:GetValue() - min;
	else
		current_time = max - statusBar:GetValue();
	end
	return format(ArcanePartyBars.TIME_LEFT, math.max(current_time,0)+0.001);
end

ArcanePartyBars.SpellcastStart = function(frame, durration, name, target)
	if (not frame) then
		frame = this;
	end
	frame.StatusBar:SetStatusBarColor(ArcanePartyBars.DefaultColors["MAIN"].r, ArcanePartyBars.DefaultColors["MAIN"].g, ArcanePartyBars.DefaultColors["MAIN"].b);
	frame.Spark:Show();
	frame.startTime = GetTime();
	frame.maxValue = frame.startTime + (durration / 1000);
	frame.StatusBar:SetMinMaxValues(frame.startTime, frame.maxValue);
	frame.StatusBar:SetValue(frame.startTime);
	frame:SetAlpha(1.0);
	frame.holdTime = 0;
	frame.casting = 1;
	frame.fadeOut = nil;
	frame:Show();
	
	local timeReadable = ArcanePartyBars.GetTimeLeft(frame.StatusBar)
	if (name) then
		frame.Name:SetText(name);
	end
	frame.Time:SetText("("..timeReadable..")");
	if (target) and (target ~= "") then
		frame.Target:SetText("<"..target..">");
	else
		frame.Target:SetText("");
	end

	frame.mode = "casting";
end

ArcanePartyBars.SpellcastStop = function(frame)
	if (not frame) then
		frame = this;
	end
	if (not frame.channeling) then
		if ( not frame:IsVisible() ) then
			frame:Hide();
		end
		if ( frame:IsShown() ) then
			frame.StatusBar:SetValue(frame.maxValue);
			frame.StatusBar:SetStatusBarColor(ArcanePartyBars.DefaultColors["SUCCESS"].r, ArcanePartyBars.DefaultColors["SUCCESS"].g, ArcanePartyBars.DefaultColors["SUCCESS"].b);
			frame.Spark:Hide();
			frame.Flash:SetAlpha(0.0);
			frame.Flash:Show();
			frame.casting = nil;
			frame.flash = 1;
			frame.fadeOut = 1;

			frame.mode = "flash";
		end
	end
end

ArcanePartyBars.SpellcastFailed = function(frame)
	if (not frame) then
		frame = this;
	end
	if ( frame:IsShown() ) then
		frame.StatusBar:SetValue(frame.maxValue);
		frame.StatusBar:SetStatusBarColor(ArcanePartyBars.DefaultColors["FAILURE"].r, ArcanePartyBars.DefaultColors["FAILURE"].g, ArcanePartyBars.DefaultColors["FAILURE"].b);
		frame.Spark:Hide();
		frame.casting = nil;
		frame.fadeOut = 1;
		frame.holdTime = GetTime() + CASTING_BAR_HOLD_TIME;
	end
end

ArcanePartyBars.SpellcastDelayed = function(frame, durration)
	if (not frame) then
		frame = this;
	end
	if( frame:IsShown() ) then
		frame.startTime = frame.startTime + (durration / 1000);
		frame.maxValue = frame.maxValue + (durration / 1000);
		frame.StatusBar:SetMinMaxValues(frame.startTime, frame.maxValue);
	end
end

ArcanePartyBars.SpellcastChannelStart = function(frame, durration, name, target)
	if (not frame) then
		frame = this;
	end

	frame.StatusBar:SetStatusBarColor(ArcanePartyBars.DefaultColors["CHANNEL"].r, ArcanePartyBars.DefaultColors["CHANNEL"].g, ArcanePartyBars.DefaultColors["CHANNEL"].b);
	frame.Spark:Hide();
	frame.maxValue = 1;
	frame.startTime = GetTime();
	frame.endTime = frame.startTime + (durration / 1000);
	frame.duration = durration / 1000;
	frame.StatusBar:SetMinMaxValues(frame.startTime, frame.endTime);
	frame.StatusBar:SetValue(frame.endTime);
	frame:SetAlpha(1.0);
	frame.holdTime = 0;
	frame.casting = nil;
	frame.channeling = 1;
	frame.fadeOut = nil;
	frame:Show();
	
	local timeReadable = ArcanePartyBars.GetTimeLeft(frame.StatusBar)
	if (name) then
		frame.Name:SetText(name);
	end
	frame.Time:SetText("("..timeReadable..")");
	if (target) and (target ~= "") then
		frame.Target:SetText("<"..target..">");
	else
		frame.Target:SetText("");
	end
end

ArcanePartyBars.SpellcastChannelUpdate = function(frame, durration)
	if (not frame) then
		frame = this;
	end
	
	if ( durration == 0 ) then
		frame.channeling = nil;
	elseif ( frame:IsShown() ) then
		local origDuration = frame.endTime - frame.startTime
		frame.endTime = GetTime() + (durration / 1000)
		frame.startTime = frame.endTime - origDuration
		--this.endTime = this.startTime + (arg1 / 1000);
		frame.StatusBar:SetMinMaxValues(frame.startTime, frame.endTime);
	end
	
	local timeReadable = ArcanePartyBars.GetTimeLeft(frame.StatusBar)
	if (name) then
		frame.Name:SetText(name);
	end
	frame.Time:SetText("("..timeReadable..")");
	if (target) and (target ~= "") then
		frame.Target:SetText("<"..target..">");
	else
		frame.Target:SetText("");
	end
end

--------------------------------------------------
-- Hook Functions For Recording Target
--------------------------------------------------

ArcanePartyBars.NoTarget = {};
ArcanePartyBars.SelfCast = {
	----------------------------------------------------------------------
	-- Spells that should be cast on player if there is no friendly target
	----------------------------------------------------------------------
	--Set any of the options to a true value to set it
	--h - this spell heals
	--n - this spell boosts mana
	--b - this spell has a buff(including heal spells that do heal over time
	--p - this spell cures poison
	--c - this spell removes curses
	--d - this spell cures disease
	--m - this spell removes magic effects
	--r - this spell must have a range specifier in the tooltip to match
	--g - this spell can only be cast on group members
	--f - this spell is a first aid spell that channels
	--l - this spell is a blessing
	--t - the texture of the buff, if it is different from the spell
	DRUID = {
		["Interface\\Icons\\Spell_Nature_NullifyPoison_02"] = {p=1;b=1;};			--Abolish Poison
		["Interface\\Icons\\Spell_Nature_NullifyPoison"] = {p=1;};						--Cure Poison
		["Interface\\Icons\\Spell_Nature_HealingTouch"] = {h=1;};							--Healing Touch
		["Interface\\Icons\\Spell_Nature_Lightning"] = {b=1;n=1;};						--Innervate
		["Interface\\Icons\\Spell_Nature_Regeneration"] = {b=1;};							--Mark of the Wild
		["Interface\\Icons\\Spell_Nature_ResistNature"] = {h=1;b=1;};					--Regrowth
		["Interface\\Icons\\Spell_Nature_Rejuvenation"] = {h=1;b=1;};					--Rejuvenation
		["Interface\\Icons\\Spell_Holy_RemoveCurse"] = {c=1;r=1};							--Remove Curse
		["Interface\\Icons\\Spell_Nature_Thorns"] = {b=1;};										--Thorns
	};
	HUNTER = {
		--["Interface\\Icons\\"] = 0;	--
	};
	MAGE = {
		["Interface\\Icons\\Spell_Holy_FlashHeal"] = {b=1;g=1;};							--Amplify Magic
		["Interface\\Icons\\Spell_Holy_MagicalSentry"] = {b=1;};							--Arcane Intellect
		["Interface\\Icons\\Spell_Nature_AbolishMagic"] = {b=1;g=1;};					--Dampen Magic
		["Interface\\Icons\\Spell_Nature_RemoveCurse"] = {c=1;};							--Remove Lesser Curse
	};
	PALADIN = {
		["Interface\\Icons\\Spell_Holy_SealOfValor"] = {b=1;};								--Blessing of Freedom
		["Interface\\Icons\\Spell_Magic_MageArmor"] = {b=1;l=1;};							--Blessing of Kings
		["Interface\\Icons\\Spell_Holy_PrayerOfHealing02"] = {b=1;l=1;};			--Blessing of Light
		["Interface\\Icons\\Spell_Holy_FistOfJustice"] = {b=1;l=1;};					--Blessing of Might
		["Interface\\Icons\\Spell_Holy_SealOfProtection"] = {b=1;g=1;l=1;};		--Blessing of Protection
		["Interface\\Icons\\Spell_Holy_SealOfSacrifice"] = {b=1;r=1;l=1;};		--Blessing of Sacrifice
		["Interface\\Icons\\Spell_Holy_SealOfSalvation"] = {b=1;g=1;l=1;};		--Blessing of Salvation
		["Interface\\Icons\\Spell_Nature_LightningShield"] = {b=1;l=1;};			--Blessing of Sanctuary
		["Interface\\Icons\\Spell_Holy_SealOfWisdom"] = {b=1;n=1;l=1;};				--Blessing of Wisdom
		["Interface\\Icons\\Spell_Holy_Renew"] = {p=1;d=1;m=1;};							--Cleanse
		["Interface\\Icons\\Spell_Nature_TimeStop"] = {b=1;g=1;};							--Divine Intervention
		["Interface\\Icons\\Spell_Holy_FlashHeal"] = {h=1;};									--Flash of Light
		["Interface\\Icons\\Spell_Holy_HolyBolt"] = {h=1;};										--Holy Light
		["Interface\\Icons\\Spell_Holy_LayOnHands"] = {h=1;};									--Lay on Hands
		["Interface\\Icons\\Spell_Holy_Purify"] = {p=1;d=1;};									--Purify
	};
	PRIEST = {
		["Interface\\Icons\\Spell_Nature_NullifyDisease"] = {p=1;b=1;};				--Abolish Disease
		["Interface\\Icons\\Spell_Holy_NullifyDisease"] = {d=1;};							--Cure Disease
		["Interface\\Icons\\Spell_Holy_DispelMagic"] = {m=1;};								--Dispel Magic (neutral)
		["Interface\\Icons\\Spell_Holy_HolyProtection"] = {b=1;};							--Divine Spirit
		["Interface\\Icons\\Spell_Holy_Excorcism"] = {b=1;};									--Fear Ward
		["Interface\\Icons\\Spell_Holy_FlashHeal"] = {h=1;};									--Flash Heal
		["Interface\\Icons\\Spell_Holy_GreaterHeal"] = {h=1;};								--Greater Heal
		["Interface\\Icons\\Spell_Holy_Heal"] = {h=1;};												--Heal
		["Interface\\Icons\\Spell_Holy_Heal02"] = {h=1;};											--Heal
		["Interface\\Icons\\Spell_Holy_LesserHeal"] = {h=1;};									--Lesser Heal
		["Interface\\Icons\\Spell_Holy_LesserHeal02"] = {h=1;};								--Lesser Heal
		["Interface\\Icons\\Spell_Holy_WordFortitude"] = {b=1;};							--Power Word: Fortitude
		["Interface\\Icons\\Spell_Holy_PowerWordShield"] = {b=1;g=1;};				--Power Word: Shield
		["Interface\\Icons\\Spell_Holy_PrayerOfFortitude"] = {b=1;};					--Prayer of Fortitude
		["Interface\\Icons\\Spell_Holy_Renew"] = {h=1;};											--Renew
		["Interface\\Icons\\Spell_Shadow_AntiShadow"] = {b=1;};								--Shadow Protection
	};
	ROGUE = {
		--["Interface\\Icons\\"] = 0;	--
	};
	SHAMAN = {
		["Interface\\Icons\\Spell_Nature_HealingWaveGreater"] = {h=1;};				--Chain Heal
		["Interface\\Icons\\Spell_Nature_RemoveDisease"] = {d=1;};						--Cure Disease
		["Interface\\Icons\\Spell_Nature_NullifyPoison"] = {p=1;};						--Cure Poison
		["Interface\\Icons\\Spell_Nature_MagicImmunity"] = {h=1;};						--Healing Wave
		["Interface\\Icons\\Spell_Nature_HealingWaveLesser"] = {h=1;};				--Lesser Healing Wave
		["Interface\\Icons\\Spell_Shadow_DemonBreath"] = {b=1;};							--Water Breathing
		["Interface\\Icons\\Spell_Frost_WindWalkOn"] = {b=1;};								--Water Walking
	};
	WARLOCK = {
		["Interface\\Icons\\Spell_Shadow_DetectInvisibility"] = {b=1;};				--Detect Invisibility and Greater Invisibility
		["Interface\\Icons\\Spell_Shadow_DetectLesserInvisibility"] = {b=1;};	--Detect Lesser Invisibility
		["Interface\\Icons\\Spell_Shadow_DemonBreath"] = {b=1;};							--Unending Breath
	};
	WARRIOR = {
		--["Interface\\Icons\\"] = 0;	--
	};
	--List of any container items that should be self cast
	Container = {
		--First Aid
		["Interface\\Icons\\INV_Misc_Bandage_15"] = {h=1;f=1;};								--Linen Bandage
		["Interface\\Icons\\INV_Misc_Bandage_18"] = {h=1;f=1;};								--Heavy Linen Bandage
		["Interface\\Icons\\INV_Misc_Bandage_14"] = {h=1;f=1;};								--Wool Bandage
		["Interface\\Icons\\INV_Misc_Bandage_17"] = {h=1;f=1;};								--Heavy Wool Bandage
		["Interface\\Icons\\INV_Misc_Bandage_01"] = {h=1;f=1;};								--Silk Bandage
		["Interface\\Icons\\INV_Misc_Bandage_02"] = {h=1;f=1;};								--Heavy Silk Bandage
		["Interface\\Icons\\INV_Misc_Bandage_19"] = {h=1;f=1;};								--Mageweave Bandage
		["Interface\\Icons\\INV_Misc_Bandage_20"] = {h=1;f=1;};								--Heavy Mageweave Bandage
		["Interface\\Icons\\INV_Misc_Bandage_11"] = {h=1;f=1;};								--Runecloth Bandage
		["Interface\\Icons\\INV_Misc_Bandage_12"] = {h=1;f=1;};								--Heavy Runecloth Bandage
		["Interface\\Icons\\INV_Misc_Slime_01"] = {p=1;};											--Anti-Venom and Strong Anti-Venom
		--Scrolls
		--Prolly too indecisive to use
		--["Interface\\Icons\\INV_Scroll_01"] = {b=1;};													--Scroll of Spirit/Intellect
		--["Interface\\Icons\\INV_Scroll_02"] = {b=1;};													--Scroll of Agility/Strength
		--["Interface\\Icons\\INV_Scroll_0t"] = {b=1;};													--Scroll of Protection/Stamina
		--Misc
		["Interface\\Icons\\INV_Misc_Herb_04"] = {h=1;};											--Sprouted Frond
	};
};

ArcanePartyBars.ScanTooltip = function(id, texture, container)
	if (container) then
		APB_Mining_Tooltip:SetBagItem(container, id);			--set tooltip to container item
	else
		APB_Mining_Tooltip:SetAction(id);				--set our tooltip to action bar item
	end
	
	local tip = Sea.wow.tooltip.scan("APB_Mining_Tooltip");	--sets tip equal to the parts of the tooltip
	if (tip and tip[1] and tip[1]["left"]) then		--Check for nil values in the spell name
		Sea.io.dprint("ArcanePartyBarDebug", "Scanning Tooltip...");
		ArcanePartyBars.spellName = tip[1]["left"];		--Set spellName(Global) from the tooltip 
		ArcanePartyBars.spellTime = "nil";			--Set spellTime(Global) = "nil" so we dont have a non string
		if (tip[2] and tip[2]["left"] and (tip[2]["left"] == SPELL_CAST_TIME_INSTANT_NO_MANA or tip[2]["left"] == SPELL_CAST_TIME_INSTANT or strfind(tip[2]["left"], ArcanePartyBars.SPELL_CAST_TIME_SEC_REGEX))) then
			ArcanePartyBars.spellTime = tip[2]["left"];
		elseif (tip[3] and tip[3]["left"] and (tip[3]["left"] == SPELL_CAST_TIME_INSTANT_NO_MANA or tip[3]["left"] == SPELL_CAST_TIME_INSTANT or strfind(tip[3]["left"], ArcanePartyBars.SPELL_CAST_TIME_SEC_REGEX))) then
			ArcanePartyBars.spellTime = tip[3]["left"];
		end
		Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - spellName: ", ArcanePartyBars.spellName, " spellTime: ", ArcanePartyBars.spellTime);
		--check if there is range on the spell otherwise it wont take a target.
		if (tip[2] and tip[2]["right"] and strfind(tip[2]["right"], ArcanePartyBars.SPELL_RANGE_REGEX)) then
			ArcanePartyBars.NoTarget[ArcanePartyBars.spellName] = nil;
			Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - Requires Target - ", tip[2]["right"]);
		elseif (tip[2] and tip[2]["left"] and strfind(tip[2]["left"], ArcanePartyBars.SPELL_RANGE_REGEX)) then
			ArcanePartyBars.NoTarget[ArcanePartyBars.spellName] = nil;
			Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - Requires Target - ", tip[2]["left"]);
		elseif (not container) and (GetActionText(id)) then
			--Slot is a macro
			ArcanePartyBars.NoTarget[ArcanePartyBars.spellName] = nil;
			Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - '", ArcanePartyBars.spellName, "' is Macro ");
		elseif (ArcanePartyBars.SelfCast["Container"][texture]) then
			ArcanePartyBars.NoTarget[ArcanePartyBars.spellName] = nil;
			Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - Requires Target - Inventory Cast");
		else
			ArcanePartyBars.NoTarget[ArcanePartyBars.spellName] = 1;
			Sea.io.dprint("ArcanePartyBarDebug", "Scan Result - No target ");
		end
		
		Sea.wow.tooltip.clear("APB_Mining_Tooltip");
		--Scan Successful.
		return true;
	end
end

ArcanePartyBars.UnitFinder = function(texture, spellName, spellTime)
	if (not spellName) then
		spellName=ArcanePartyBars.spellName;
	else
		ArcanePartyBars.spellName=spellName;
	end
	if (not spellTime) then
		spellTime=ArcanePartyBars.spellTime;
	elseif (spellTime == "1") then
		spellTime = SPELL_CAST_TIME_INSTANT_NO_MANA;
	else
		spellTime="nil";
	end
	ArcanePartyBars.isSpellInQueue = 1;
	ArcanePartyBars.SavedTarget = nil;
	if (ArcanePartyBars.NoTarget[spellName]) then
		ArcanePartyBars.isNoTargetCast = 1;
		return;
	else
		ArcanePartyBars.isNoTargetCast = nil;
	end
	
	local _, playerClass = UnitClass("player");
	
	--this handles anyspell when the player has a target and the spell already target someone
	if (not SpellIsTargeting()) then
		if (UnitExists("target")) then
			--casting on a friend
			if (UnitIsFriend("target","player")) then	
				Sea.io.dprint("ArcanePartyBarDebug", "Instacast fired friend");
				ArcanePartyBars.SavedTarget = UnitName("target");
			else
				Sea.io.dprint("ArcanePartyBarDebug", "Instacast fired nonfriend");
				if (ArcanePartyBars.SelfCast[playerClass][texture]) then
					--A healing spell is being cast on a enemy? The player must have selfCast
					ArcanePartyBars.SavedTarget = UnitName("player");
				else
					--A non heal must be on that target then
					ArcanePartyBars.SavedTarget = UnitName("target");
				end
			end
		elseif (spellTime == SPELL_CAST_TIME_INSTANT_NO_MANA or spellTime == SPELL_CAST_TIME_INSTANT) then
			--instant casts dont need to check for when the spell casts
			if (ArcanePartyBars.SelfCast[playerClass][texture]) then
				--the spell can be cast on the player
				ArcanePartyBars.SavedTarget = nil;
			end
		end
	else
		--SpellIsTargeting and then a different Spell is cast
		ArcanePartyBars.SavedTarget = UnitName("target");
	end
end

ArcanePartyBars.UseContainerItem_Hook = function(container, id)
	Sea.io.dprint("ArcanePartyBarDebug", "UseContainerItem triped! container: ", container, " id: ", id);
	local texture = GetContainerItemInfo(container, id);
	if (ArcanePartyBars.ScanTooltip(id, texture, container)) then
		ArcanePartyBars.UnitFinder(texture);
	end
end

ArcanePartyBars.CastSpell_Hook = function(id, book)
	local spell, rank = GetSpellName(id, book); 
	Sea.io.dprint("ArcanePartyBarDebug", "CastSpell = ", spell, " Rank = ", rank); 
	ArcanePartyBars.spellName = spell;
	ArcanePartyBars.spellTime = "";
	--Check NoTarget
	local texture = GetSpellTexture(id, book);
	ArcanePartyBars.UnitFinder(texture);
end

ArcanePartyBars.UseAction_Hook = function(id, number, onSelf)
	Sea.io.dprint("ArcanePartyBarDebug", "UseAction triped! id: ", id);
	local texture = GetActionTexture(id);
	if (ArcanePartyBars.ScanTooltip(id, texture, container)) then
		ArcanePartyBars.UnitFinder(texture);
	end
	if (onSelf) then
		ArcanePartyBars.SavedTarget = UnitName("player");
	end
end

ArcanePartyBars.CameraOrSelectOrMoveStop_Hook = function()
	-- trying to parse out name... (remember: corpse of ...)
	if (SpellIsTargeting() and GameTooltip:IsVisible() ) then
		local field = getglobal("GameTooltipTextLeft1");
		if( field and field:IsVisible() ) then
			local name = field:GetText();
			Sea.io.dprint("ArcanePartyBarDebug", "Cameraorselectormovestop name= "..name);
			 -- remove "Corpse of " from name...
			local tempName = gsub(name, ArcanePartyBars.CORPSE_TOOLTIP_REGEX, "%1");
			if (tempName) then
				-- casting on a corpse...
				name = tempName;
			end
			ArcanePartyBars.SavedTarget = name;
		end
	end
end

--Used to hook TargetUnit, SpellTargetUnit and CastOptions.CastAtUnit
ArcanePartyBars.TargetUnit_Hook = function(unit)
	if (unit) then 
		Sea.io.dprint("ArcanePartyBarDebug", "Hooked target unit = ".. unit);
		ArcanePartyBars.SavedTarget = UnitName(unit);
	else 
		Sea.io.dprint("ArcanePartyBarDebug", "Hooked target unit but no unit!?!"); 
	end
end

ArcanePartyBars.TargetByName_Hook = function(name)
	if (name) then 
		Sea.io.dprint("ArcanePartyBarDebug", "Hooked TargetByName = ".. name);
		ArcanePartyBars.SavedTarget = name;
	else 
		Sea.io.dprint("ArcanePartyBarDebug", "Hooked TargetByName but no name!?!"); 
	end
end

ArcanePartyBars.TargetLastTarget_Hook = function()
	if (ArcanePartyBars.lastTarget) then
		Sea.io.dprint("ArcanePartyBarDebug", "Hooked TargetLastTarget = "..ArcanePartyBars.lastTarget);
		ArcanePartyBars.SavedTarget = ArcanePartyBars.lastTarget;
	end
end

--Used to hook TargetNearestFriend, TargetNearestPartyMember, TargetNearestRaidMember 
ArcanePartyBars.TargetUnknown_Hook = function()
	Sea.io.dprint("ArcanePartyBarDebug", "Hooked TargetUnknown");
	ArcanePartyBars.SavedTarget = nil;
end


--------------------------------------------------
-- ArcanePartyBarFrame Functions
--------------------------------------------------

ArcanePartyBars.emptyfunc = function()
end

ArcanePartyBars.MasterOnLoad = function(frame)
	if (not frame) then
		frame = this;
	end
	ArcanePartyBars.Register(frame);
	ArcanePartyBars.RegisterForSkyMessages();
	if (Hx_OnSpellCast) then
		--Hijack Healix!
		Sea.util.hook("Hx_OnSpellCast", "ArcanePartyBars.emptyfunc", "hide");
		ArcanePartyBars.Hx_OnSpellCast = Sea.util.Hooks.Hx_OnSpellCast.orig;
	end
	if (MCom) then
		ArcanePartyBars.RegisterForMCom();
	end
end

ArcanePartyBars.MasterOnEvent = function(frame)
	if (not frame) then
		frame = this;
	end
	if ( event == "SPELLCAST_START" ) then
		--arg1: Name
		--arg2: cast time
		Sea.io.dprint("ArcanePartyBarDebug", "SPELLCAST_START isNoTargetCast: ", ArcanePartyBars.isNoTargetCast, " SavedTarget: ", ArcanePartyBars.SavedTarget, " isSpellInQueue: ", ArcanePartyBars.isSpellInQueue);
		local target;
		if (ArcanePartyBars.isNoTargetCast) then
			target = "";
			ArcanePartyBars.isNoTargetCast = nil;
		elseif (ArcanePartyBars.SavedTarget) then 
			target = ArcanePartyBars.SavedTarget;
			ArcanePartyBars.SavedTarget = nil;
		else
			target = UnitName("player"); 
		end	
		Sea.io.dprint("ArcanePartyBarDebug", arg1, arg2, target);
		if (ArcanePartyBars.Hx_OnSpellCast) and (arg1) and (arg2) then
			ArcanePartyBars.Hx_OnSpellCast(arg1, (arg2/1000).."secs", target); 
		end 
		--[[
		if (type(tonumber(arg2)) == "number") then
			--Acount for Upload Time
			local down, up, lag = GetNetStats();
			arg2 = tonumber(arg2) - up*1000;
		end
		]]--
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "t,"..arg2..","..arg1..","..target, SKY_PARTY, "AP" );
		end
		ArcanePartyBars.isSpellInQueue = nil;
	elseif ( event == "SPELLCAST_STOP" ) then
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "s,,,", SKY_PARTY, "AP" );
		end
		if (ArcanePartyBars.Hx_OnSpellCast) and (ArcanePartyBars.isSpellInQueue) and (ArcanePartyBars.spellTime == SPELL_CAST_TIME_INSTANT_NO_MANA or ArcanePartyBars.spellTime == SPELL_CAST_TIME_INSTANT) then
			local target;
			if (ArcanePartyBars.isNoTargetCast) then
				target = "";
				ArcanePartyBars.isNoTargetCast = nil;
			elseif (ArcanePartyBars.SavedTarget) then 
				target = ArcanePartyBars.SavedTarget;
				ArcanePartyBars.SavedTarget = nil;
			else
				target = UnitName("player"); 
			end	
			ArcanePartyBars.Hx_OnSpellCast(ArcanePartyBars.spellName,ArcanePartyBars.spellTime, target); 
		end
		ArcanePartyBars.SavedTarget = nil; 
		ArcanePartyBars.isSpellInQueue = nil;
	elseif ( event == "SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED" ) then
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "f,,,", SKY_PARTY, "AP" );
		end
		ArcanePartyBars.isSpellInQueue = nil;
		--ArcanePartyBars.SavedTarget = nil;
	elseif ( event == "SPELLCAST_DELAYED" ) then
		--arg1: delayed time
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "d,"..arg1..",,", SKY_PARTY, "AP" );
		end
	elseif ( event == "SPELLCAST_CHANNEL_START" ) then
		--arg1: cast time
		--arg2: "Channeling"
		Sea.io.dprint("ArcanePartyBarDebug", "SPELLCAST_CHANNEL_START isNoTargetCast: ", ArcanePartyBars.isNoTargetCast, " SavedTarget: ", ArcanePartyBars.SavedTarget, " isSpellInQueue: ", ArcanePartyBars.isSpellInQueue);
		local target;
		if (ArcanePartyBars.isNoTargetCast) then
			target = "";
			ArcanePartyBars.isNoTargetCast = nil;
		elseif (ArcanePartyBars.SavedTarget) then 
			target = ArcanePartyBars.SavedTarget;
			ArcanePartyBars.SavedTarget = nil;
		else
			target = UnitName("player"); 
		end	
		Sea.io.dprint("ArcanePartyBarDebug", arg1, arg2, ArcanePartyBars.spellName, target);
		if (ArcanePartyBars.Hx_OnSpellCast) and (arg1) and (arg2) then
			ArcanePartyBars.Hx_OnSpellCast(ArcanePartyBars.spellName, (arg1/1000).." sec cast", target); 
		end 
		--[[
		if (type(tonumber(arg1)) == "number") then
			--Acount for Upload Time
			local down, up, lag = GetNetStats();
			arg1 = tonumber(arg1) - up*1000;
		end
		]]--
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "c,"..arg1..","..ArcanePartyBars.spellName..","..target, SKY_PARTY, "AP" );
		end
		ArcanePartyBars.isSpellInQueue = nil;
	elseif ( event == "SPELLCAST_CHANNEL_UPDATE" ) then
		--arg1: delayed time
		if (UnitInParty("party1")) and (ArcanePartyBars_SendCastingEvents) and (Sky.isChannelActive(SKY_PARTY)) then
			Sky.sendAlert( "u,"..arg1..",,", SKY_PARTY, "AP" );
		end
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		ArcanePartyBars.isSpellInQueue = nil;
		if (ArcanePartyBars.currentTarget) and (ArcanePartyBars.currentTarget ~= UnitName("target")) then
			ArcanePartyBars.lastTarget = ArcanePartyBars.currentTarget;
		elseif (not ArcanePartyBars.lastTarget) then
			ArcanePartyBars.lastTarget = UnitName("target");
		end
		ArcanePartyBars.currentTarget = UnitName("target");
		if (ArcanePartyBars.Hx_OnSpellCast) and (ArcanePartyBars.isSpellInQueue) and (UnitExists("target")) and (not SpellIsTargeting()) then 
			ArcanePartyBars.Hx_OnSpellCast(ArcanePartyBars.spellName,ArcanePartyBars.spellTime,UnitName("target")); 
		end
	elseif ( event == "PARTY_MEMBERS_CHANGED" ) then
		ArcanePartyBars.AttemptToJoinSkyParty();
	elseif ( event == "VARIABLES_LOADED" ) then
	end
end

--------------------------------------------------
-- Sky Addon Registration
--------------------------------------------------


ArcanePartyBars.RegisterForSkyMessages = function()
	Sky.registerAlert( 
		{
			id = "AP",
			func = ArcanePartyBars.SkyAlertProccessing,
			description = "Alerts for Arcane Party Bar Casts",
		}
	);
end

ArcanePartyBars.UnregisterForSkyMessages = function()
	Sky.unregisterAlert("AP");
end

ArcanePartyBars.SkyAlertProccessing = function(msg, sender, system, time)
	if ( type(msg) == "string" ) and (strlen(msg) > 0) then
		Sea.io.dprint("ArcanePartyBarDebug", msg);
		local _, _, key, count, name, target = string.find(msg, "([tdfscu]),(.*),(.*),(.*)");
		if (key) and (count) and (name) and (target) then
			--[[
			if (type(tonumber(count)) == "number") then
				--Acount for Downlaod Time
				local down, up, lag = GetNetStats();
				count = tonumber(count) - down*1000;
			end
			]]--
			local bar = ArcanePartyBars.GetPartyMembersArcaneBar(sender);
			if (bar) then
				if (key == "t") then
					ArcanePartyBars.SpellcastStart(bar, tonumber(count), name, target);
					Sea.io.dprint("ArcanePartyBarDebug", sender.." Casting: ".. name..".");
				elseif (key == "d") then
					ArcanePartyBars.SpellcastDelayed(bar, tonumber(count));
					Sea.io.dprint("ArcanePartyBarDebug", sender.."'s cast is being delayed.");
				elseif (key == "f") then
					ArcanePartyBars.SpellcastFailed(bar);
					Sea.io.dprint("ArcanePartyBarDebug", sender.."'s cast has failed.");
				elseif (key == "s") then
					ArcanePartyBars.SpellcastStop(bar);
					Sea.io.dprint("ArcanePartyBarDebug", sender.."'s cast has ended.");
				elseif (key == "c") then
					ArcanePartyBars.SpellcastChannelStart(bar, tonumber(count), name, target);
					Sea.io.dprint("ArcanePartyBarDebug", sender.." Channeling: ".. name..".");
				elseif (key == "u") then
					ArcanePartyBars.SpellcastChannelUpdate(bar, tonumber(count));
					Sea.io.dprint("ArcanePartyBarDebug", sender.." Channeling '".. name.."' Updated To: ".. count..".");
				end
			end
		end
	end
end

ArcanePartyBars.GetPartyMembersArcaneBar = function(name)
	if (ArcanePartyBarDebugParty1) then
		--Show your casts on party1.
		return ArcanePartyBar1;
	else
		for i=1, 4 do
			if (UnitName("party"..i) == name) then
				return getglobal("ArcanePartyBar"..i);
			end
		end
	end
end

--------------------------------------------------
-- Menu (From MobileFrames)
--------------------------------------------------

ArcanePartyBars.LoadDropDownMenu = function()
	if (not UIDROPDOWNMENU_MENU_VALUE) then
		return;
	end
	--Title
	local info = {};
	info.text = ArcanePartyBars.localization.BAR_TITLES[UIDROPDOWNMENU_MENU_VALUE];
	info.notClickable = 1;
	info.isTitle = 1;
	UIDropDownMenu_AddButton(info, 1);
	
	--Reset
	local info = {};
	info.text = RESET;
	info.value = "Reset";
	info.func = function() ArcanePartyBars.Reset(getglobal("ArcanePartyBar"..UIDROPDOWNMENU_MENU_VALUE)); end;
	if (getglobal("ArcanePartyBar"..UIDROPDOWNMENU_MENU_VALUE)) and (not getglobal("ArcanePartyBar"..UIDROPDOWNMENU_MENU_VALUE):IsUserPlaced()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info, 1);
	
	--Reset All
	local info = {};
	info.text = RESET_ALL;
	info.value = "ResetAll";
	info.func = ArcanePartyBars.ResetAll;
	UIDropDownMenu_AddButton(info, 1);
	
	ArcanePartyBars.DropDownReposition(getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL), this:GetName());
end

ArcanePartyBars.DropDownOnLoad = function()
	UIDropDownMenu_Initialize(this, ArcanePartyBars.LoadDropDownMenu, "MENU");
end

ArcanePartyBars.DropDownReposition = function(listFrame, anchorName)
	local customPoint = "TOPLEFT";   --Default Anchor
	local offscreenY, offscreenX, anchorPoint, relativePoint, offsetX, offsetY;
	-- Determine whether the menu is off the screen or not
	local offscreenY, offscreenX;
	--Hack for built in bug
	if ( not listFrame:GetRight() ) then
		return;
	end
	
	if ( listFrame:GetBottom() < WorldFrame:GetBottom() ) then
		offscreenY = 1;
	end
	if ( listFrame:GetRight() > WorldFrame:GetRight() ) then
		offscreenX = 1;
	end
	
	local anchorPoint, relativePoint, offsetX, offsetY;
	if ( offscreenY == 1 ) then
		if ( offscreenX == 1 ) then
			anchorPoint = string.gsub(customPoint, "TOP(.*)", "BOTTOM%1");
			anchorPoint = string.gsub(anchorPoint, "(.*)LEFT", "%1RIGHT");
			relativePoint = "TOPRIGHT";
			offsetX = 0;
			offsetY = 14;
		else
			anchorPoint = string.gsub(customPoint, "TOP(.*)", "BOTTOM%1");
			relativePoint = "TOPLEFT";
			offsetX = 0;
			offsetY = 14;
		end
	else
		if ( offscreenX == 1 ) then
			anchorPoint = string.gsub(customPoint, "(.*)LEFT", "%1RIGHT");
			relativePoint = "BOTTOMRIGHT";
			offsetX = 0;
			offsetY = -14;
		else
			anchorPoint = customPoint;
			relativePoint = "BOTTOMLEFT";
			offsetX = 0;
			offsetY = -14;
		end
	end
	listFrame:ClearAllPoints();
	listFrame:SetPoint(anchorPoint, anchorName, relativePoint, offsetX, offsetY);
	
	-- Reshow the "MENU" border
	if (getglobal(listFrame:GetName().."Backdrop")) and (getglobal(listFrame:GetName().."MenuBackdrop")) then
		getglobal(listFrame:GetName().."Backdrop"):Hide();
		getglobal(listFrame:GetName().."MenuBackdrop"):Show();
	end

end

--------------------------------------------------
-- Configuration Registeration
--------------------------------------------------

ArcanePartyBars.GetLocal = function(index)
	return ArcanePartyBars.localization[index];
end

ArcanePartyBars.GetInfoLocal = function(index)
	return ArcanePartyBars.localization[index].."\n"..ArcanePartyBars.localization.info[index];
end

ArcanePartyBars.RegisterForMCom = function()
	
	local optionSet = {};
	
	--------------------------------------------------
	-- Presets Registering
	--------------------------------------------------
	
	table.insert(optionSet, {
		id="OptionsHeader";
		text=ArcanePartyBars.GetLocal("OPTIONS_HEADER");
		helptext=ArcanePartyBars.GetInfoLocal("OPTIONS_HEADER");
		type=K_HEADER;
		difficulty=1;
	});
	
	table.insert(optionSet, {
		id="Reset";
		type=K_BUTTON;
		text=ArcanePartyBars.GetLocal("RESET_BARS");
		helptext=ArcanePartyBars.GetInfoLocal("RESET_BARS");
		callback=ArcanePartyBars.ResetAll;
		setup={buttonText=RESET_ALL};
		mcopts = {
			subcom = string.lower("reset");
		};
	});
	
	local vars = { "ArcanePartyBars_TopText", "ArcanePartyBars_Mobile", "ArcanePartyBars_AutoJoinSkyParty", "ArcanePartyBars_SendCastingEvents", "ArcanePartyBars_ShowPartyCasting", "ArcanePartyBars_SimulateOnUpdate" };
	
	for index, varString in vars do
		table.insert(optionSet, {
			id = ArcanePartyBars.ConfigTable[varString].id;
			check = true;
			type = K_TEXT;
			difficulty = 1;
			text = ArcanePartyBars.GetLocal(ArcanePartyBars.ConfigTable[varString].text);
			helptext = ArcanePartyBars.GetInfoLocal(ArcanePartyBars.ConfigTable[varString].text);
			difficulty = ArcanePartyBars.ConfigTable[varString].difficulty;
			default = {checked = ArcanePartyBars.ConfigTable[varString].default;};
			disabled = {checked = ArcanePartyBars.ConfigTable[varString].disabled;};
			mcopts = {
				subcom = ArcanePartyBars.ConfigTable[varString].subcom;
				varbool = varString;
				update = function(varName) ArcanePartyBars[ArcanePartyBars.ConfigTable[varName][getglobal(varName)]](); end;
				--noupdate = function(varName) ArcanePartyBars[ArcanePartyBars.ConfigTable[varName][getglobal(varName)]](); end;
			};
		});
	end
	
	table.insert(optionSet,  {
		id = "SimulationPeriod";
		type = K_SLIDER;
		difficulty = 4;
		text = ArcanePartyBars.GetLocal("SIM_PERIOD");
		helptext = ArcanePartyBars.GetInfoLocal("SIM_PERIOD");
		dependencies = {["SimulateOnUpdate"]={checked=true}};
		default = {slider=.07};
		disabled = {slider=.07};
		setup = {
			sliderMin = .01;
			sliderMax = .25;
			sliderStep = .01;
			sliderText = ArcanePartyBars.GetLocal("SIM_PERIOD_TEXT");
		};
		mcopts = {
			subcom = "simperiod";
			varnum = "ArcanePartyBars_OnUpdateSimulationPeriod";
		};
	});
	
	--------------------------------------------------
	-- Config Set Registering
	--------------------------------------------------
			
	MCom.registerSmart(
		{
			supercom = ArcanePartyBars.ConfigTable["ArcanePartyBars_Enabled"].com;
			uifolder = "combat",
			uiset = {
				id = ArcanePartyBars.ConfigTable["ArcanePartyBars_Enabled"].id;
				text = ArcanePartyBars.GetLocal(ArcanePartyBars.ConfigTable["ArcanePartyBars_Enabled"].text);
				helptext = ArcanePartyBars.GetInfoLocal(ArcanePartyBars.ConfigTable["ArcanePartyBars_Enabled"].text);
				difficulty = ArcanePartyBars.ConfigTable["ArcanePartyBars_Enabled"].difficulty;
				default = true;
				options = optionSet;
			}
		}
	);
	
end

--/z ArcanePartyBar1StatusBarName:SetWidth(94)
--/z ArcanePartyBarDebugParty1=true;
--/z ArcanePartyBars.ToggleOnUpdateSimulation();
--/z PartyMemberFrame1:Show();PartyMemberFrame1Name:SetText("Anduinlothar");ArcanePartyBars.SpellcastChannelStart(ArcanePartyBar1, 2000, "Corruption", "Bob")
--/z PartyMemberFrame1:Show();PartyMemberFrame1Name:SetText("Anduinlothar");ArcanePartyBars.Show()
--/z ArcanePartyBars.SpellcastStop(ArcanePartyBar1)
--/z ArcanePartyBars.SpellcastChannelStart(ArcanePartyBar1, 10000, "Corruption", "Bob")
--/z ArcanePartyBar1Flash:SetWidth(160);ArcanePartyBar1StatusBarBorder:SetHeight(18);ArcanePartyBars.SpellcastStart(ArcanePartyBar1, 2000, "Corruption", "Bob")
--/z ArcanePartyBar1:ClearAllPoints();ArcanePartyBar1:SetPoint("LEFT", "PartyMemberFrame1", "RIGHT", 7, 0);ArcanePartyBars.SpellcastStart(ArcanePartyBar1, 2000, "Corruption", "Bob")
--/z ArcanePartyBar1StatusBarName:ClearAllPoints();ArcanePartyBar1StatusBarName:SetPoint("LEFT", "ArcanePartyBar1StatusBar", "RIGHT", 7, 1);ArcanePartyBars.SpellcastStart(ArcanePartyBar1, 2000, "Corruption", "Bob")
--/z ArcanePartyBar1StatusBarTime:ClearAllPoints();ArcanePartyBar1StatusBarTime:SetPoint("LEFT", "ArcanePartyBar1StatusBarName", "RIGHT", 3, 0)
--/z ArcanePartyBars.SpellcastStart(ArcanePartyBar1, 2000)
--/z ArcanePartyBars.SpellcastStop(ArcanePlayerBar1)
--/z ArcanePartyBars.SpellcastDelayed(ArcanePlayerBar1, 1000)
--/z ArcanePlayerBar1

